//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using SolidBrush = MindFusion.Drawing.SolidBrush;


namespace MindFusion.Diagramming.WinForms.Samples.CS.CustomDraw
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			diagramView.SmoothingMode = SmoothingMode.AntiAlias;
			diagramView.Behavior = Behavior.DrawLinks;

			diagram.ShapeHandlesStyle = HandlesStyle.Invisible;
			diagram.DynamicLinks = true;
			diagram.DefaultShape = Shapes.Ellipse;
			diagram.AllowSelfLoops = false;

			diagram.Clicked += diagram_Clicked;
			diagram.LinkCreated += diagram_LinkCreated;
			diagram.DrawLink += diagram_DrawLink;

			comboBox1.SelectedIndex = 0;
		}

		private void diagram_DrawLink(object sender, DrawLinkEventArgs e)
		{
			if (e.Shadow)
			{
				// We are not drawing arrows shadows
				return;
			}

			PointF a = e.Points[0];
			PointF b = e.Points[e.Points.Count - 1];

			if (a.Equals(b))
			{
				// 0-length arrow, skip drawing
				return;
			}

			// If everything goes as planned, the arrow's type
			// should be found in the arrow's tag (where we put it
			// when the arrow was created).
			// There is a good chance that the arrow is not yet
			// considered created if the arrow is still being
			// dragged by the used from one box to another. In this case
			// consider the arrow's type to be Flat.
			ArrowType type = ArrowType.Flat;
			if (e.Link.Tag is ArrowType)
				type = (ArrowType)e.Link.Tag;

			// The fill color of the origin
			Color oc = e.Link.Origin == null ?
				Color.Black : (e.Link.Origin.Brush as SolidBrush).Color;
			// The fill color of the destination. The destination
			// can be null if the arrow is not yet created
			Color dc = e.Link.Destination == null ?
				Color.Black : (e.Link.Destination.Brush as SolidBrush).Color;

			// In order to display a rectangle instead of a simple line,
			// we need to find four points. For this purpose we
			// perform the following transformations
			float an = 0, r = 0;
			PointF p1 = PointF.Empty;
			PointF p2 = PointF.Empty;
			PointF p3 = PointF.Empty;
			PointF p4 = PointF.Empty;

			// Find two points around the origin control point
			Convert.CartesianToPolar(a, b, ref an, ref r);
			Convert.PolarToCartesian(a, an + 105, 1.2f, ref p1);
			Convert.PolarToCartesian(a, an - 105, 1.2f, ref p2);

			// At this point in 'r' we have the length of the arrow
			// expressed in the current graphics unit (millimeters by default),
			// and in 'an' we have the angle, between the arrow line and
			// one of the coordinate axis
			float length = r;
			float angle = an;

			// Find two points around the destination control point
			Convert.CartesianToPolar(b, a, ref an, ref r);
			Convert.PolarToCartesian(b, an + 105, 1.2f, ref p3);
			Convert.PolarToCartesian(b, an - 105, 1.2f, ref p4);

			PointF[] pts = new PointF[]
				{
					p1, p2, p3, p4, p1
				};

			// Create the graphics path containing the arrow rectangle
			GraphicsPath path = new GraphicsPath(FillMode.Winding);
			path.AddLines(pts);

			// Solid fill the arrow with white color, or the color
			// of the destination box, if the arrow's type is gradient
			Brush solid = new System.Drawing.SolidBrush(
				type == ArrowType.Gradient ? dc : Color.White);
			e.Graphics.FillPath(solid, path);

			// The frame pen
			Pen frame = e.Link.EffectivePen.CreateGdiPen(e.Graphics);

			// We have to calculate the progress bar
			float progress = e.Link.Weight;

			// Calculate the progress bar lenght
			float progLen = progress * length / 100;

			// Now we have to find two points which
			// define the end of the progress bar
			// depending on the progress length
			PointF pp1 = PointF.Empty;
			PointF pp2 = PointF.Empty;
			PointF pp = PointF.Empty;

			Convert.PolarToCartesian(a, angle, progLen, ref pp);
			Convert.PolarToCartesian(pp, angle - 105, 1.2f, ref pp1);
			Convert.PolarToCartesian(pp, angle + 105, 1.2f, ref pp2);

			// Now depending on the arrow type perform some specific
			// drawing
			switch (type)
			{
			case ArrowType.Flat:
				{
					// Create the graphics path containing the rectangle of
					// the progress bar
					GraphicsPath ppath = new GraphicsPath();
					ppath.AddLines(new PointF[]
						{
							pp1, pp2, p1, p2
						});

					// Fill the path with the color of the origin box
					Brush brush = new System.Drawing.SolidBrush(oc);
					e.Graphics.FillPath(brush, ppath);
					brush.Dispose();

					// Frame the progress bar
					e.Graphics.DrawPath(frame, ppath);

					ppath.Dispose();
				}
				break;

			case ArrowType.Arrow:
				{
					// Find the arrow point
					PointF arrowPoint = PointF.Empty;
					Convert.PolarToCartesian(a, angle, progLen + 3, ref arrowPoint);

					// Create the graphics path containing the rectangle of
					// the progress bar
					GraphicsPath ppath = new GraphicsPath();
					ppath.AddLines(new PointF[]
						{
							pp1, arrowPoint, pp2, p1, p2
						});

					// Fill the path with the color of the origin box
					Brush brush = new System.Drawing.SolidBrush(oc);
					e.Graphics.FillPath(brush, ppath);
					brush.Dispose();

					// Frame the progress bar
					e.Graphics.DrawPath(frame, ppath);

					ppath.Dispose();
				}
				break;

			case ArrowType.Gradient:
				{
					// Create the graphics path containing the rectangle of
					// the progress bar
					GraphicsPath ppath = new GraphicsPath();
					ppath.AddLines(new PointF[]
						{
							pp1, pp2, p1, p2
						});

					// Fill the path with the gradient color
					Brush brush = new LinearGradientBrush(ppath.GetBounds(), dc, oc, 180 - angle);
					e.Graphics.FillPath(brush, ppath);
					brush.Dispose();

					// Do no draw frame

					ppath.Dispose();
				}
				break;

			default:
				break;
			}

			// Draw the arrow frame using the arrow's pen settings
			e.Graphics.DrawPath(frame, path);

			// Note: Instead of framing the whole rectangle we can only
			// draw the two parallel lines (results in cool effect).
			// To see it, comment the above line of code and
			// uncomment the two lines below
			// e.Graphics.DrawLine(frame, p1, p4);
			// e.Graphics.DrawLine(frame, p2, p3);

			// Dispose the GDI objects
			frame.Dispose();
			solid.Dispose();
			path.Dispose();
		}

		private Random _r = new Random();
        int[,] colorCodes = new int[10, 3] { 
        {224, 233, 233},
        {192, 192, 192},
        {102, 154, 204},
        {156, 170, 198},
        {206, 0, 0},
        {90, 121, 165},
        {97, 106, 127},
        {0, 52, 102},
        {45, 57, 86},
        {0, 0, 99}};

		private void diagram_Clicked(object sender, DiagramEventArgs e)
		{
			PointF location = e.MousePosition;
			location.X -= 8;
			location.Y -= 8;

			SizeF size = new SizeF(15, 15);

			ShapeNode shape = diagram.Factory.CreateShapeNode(location, size);
            int index = _r.Next(9);
			shape.Brush = new SolidBrush(Color.FromArgb(
                colorCodes[index, 0], colorCodes[index, 1], colorCodes[index, 2]));
		}

		private void diagram_LinkCreated(object sender, LinkEventArgs e)
		{
			// Zero the progress
			// (the progress is considered to vary from 0 to 100 in the sample)
			e.Link.Weight = 0;

			// Set progress type
			e.Link.Tag = (ArrowType)comboBox1.SelectedIndex;
		}

		private Timer _t = null;

		private void button1_Click(object sender, EventArgs e)
		{
			_t = new Timer();
			_t.Interval = 200;
			_t.Tick += new EventHandler(_t_Tick);
			_t.Start();
		}

		private void _t_Tick(object sender, EventArgs e)
		{
			foreach (DiagramLink link in diagram.Links)
			{
				link.Weight += _r.Next(5);
				if (link.Weight > 100)
					link.Weight = 100;
			}

			diagram.Invalidate();
		}
	}

	/// <summary>
	/// Defines the custom-drawn arrow type.
	/// </summary>
	internal enum ArrowType
	{
		Flat,
		Arrow,
		Gradient
	}

	/// <summary>
	/// Provides various conversions.
	/// </summary>
	internal class Convert
	{
		/// <summary>
		/// Converts polar coordinates to the corresponding
		/// Cartesian coordinates, using the specified point as
		/// a center of the coordinate system.
		/// </summary>
		public static void PolarToCartesian(PointF coordCenter,
			float a, float r, ref PointF cartesian)
		{
			if (r == 0)
			{
				cartesian = coordCenter;
				return;
			}

			cartesian.X = (float)(coordCenter.X + Math.Cos(a * Math.PI / 180) * r);
			cartesian.Y = (float)(coordCenter.Y - Math.Sin(a * Math.PI / 180) * r);
		}

		/// <summary>
		/// Converts Cartesian coordinates to the corresponding
		/// polar coordinates, using the specified point as
		/// a center of the coordinate system.
		/// </summary>
		public static void CartesianToPolar(PointF coordCenter,
			PointF cartesian, ref float a, ref float r)
		{
			if (coordCenter == cartesian)
			{
				a = 0;
				r = 0;
				return;
			}

			float dx = cartesian.X - coordCenter.X;
			float dy = cartesian.Y - coordCenter.Y;
			r = (float)(Math.Sqrt(Math.Pow(dx, 2) + Math.Pow(dy, 2)));

			a = (float)(Math.Atan(-dy / dx) * 180 / Math.PI);
			if (dx < 0) a += 180;
		}
	}
}